
# q-expansion of newform 2366.2.d.g, downloaded from the LMFDB on 22 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2366
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[339, [1, 0]], [2199, [-1, 0]]]
aps_data = [[0, 1], [1, 0], [0, -4], [0, -1], [0, -1], [0, 0], [-4, 0], [0, -2], [7, 0], [-8, 0], [0, -3], [0, 7], [0, 7], [8, 0], [0, 3], [0, 0], [0, -6], [-13, 0], [0, -7], [0, -4], [0, 9], [-13, 0], [0, 16], [0, -6], [0, -11], [-9, 0], [-10, 0], [12, 0], [0, -14], [1, 0], [-13, 0], [0, 0], [0, -14], [-20, 0], [0, -15], [0, -4], [7, 0], [0, 12], [0, 0], [-14, 0], [18, 0], [-13, 0], [24, 0], [0, -4], [0, -3], [-16, 0], [22, 0], [0, -9], [0, -28], [0, -20], [13, 0], [0, 6], [0, -10], [-17, 0], [12, 0], [16, 0], [3, 0], [0, -3], [14, 0], [0, -20], [-17, 0], [0, 14], [0, -12], [30, 0], [6, 0], [0, 13], [0, -15], [-9, 0], [-32, 0], [0, 2], [0, 11], [0, 0], [12, 0], [-32, 0], [0, 8], [0, -21], [-8, 0], [0, 8], [0, 8], [0, 10], [-19, 0], [0, 11], [0, -2], [18, 0], [2, 0], [0, 0], [0, 6], [0, -18], [0, -16], [0, -12], [20, 0], [0, 0], [0, -26], [-40, 0], [0, -37], [-16, 0], [0, 18], [-30, 0], [-15, 0], [0, -38], [-4, 0], [0, 31], [31, 0], [-29, 0], [-26, 0], [0, 18], [0, 18], [0, -10], [27, 0], [2, 0], [14, 0], [0, -17], [0, 38], [0, 14], [0, 22], [25, 0], [0, 28], [48, 0], [0, 0], [6, 0], [0, -8], [33, 0], [13, 0], [0, 31], [0, 20], [0, 0], [0, -51], [22, 0], [-18, 0], [0, 12], [0, -8], [0, 44], [-37, 0], [-2, 0], [0, 1], [0, -25], [0, -38], [0, -40], [-21, 0], [30, 0], [0, -20], [0, -46], [35, 0], [0, 4], [18, 0], [0, 21], [0, -44], [-42, 0], [31, 0], [0, 24], [0, -29], [14, 0], [34, 0], [14, 0], [-54, 0], [-12, 0], [-37, 0], [0, 27], [-48, 0], [0, -14], [0, -8], [34, 0], [0, -18], [-45, 0], [0, -30], [0, 32], [-33, 0], [31, 0], [0, 52], [22, 0], [0, -44], [0, 30], [3, 0], [0, -36], [42, 0], [-12, 0], [0, 9], [0, 30], [-20, 0], [-55, 0], [0, -34], [-19, 0], [-22, 0], [0, 25], [0, -36], [-54, 0], [-24, 0], [0, -4], [0, -4], [0, -25], [64, 0], [0, 13], [-54, 0], [0, -30], [10, 0], [-30, 0], [0, 40], [54, 0], [0, -22], [-12, 0], [0, 21], [8, 0], [0, -36], [10, 0], [0, -10], [-34, 0], [0, -28], [54, 0], [0, 42], [31, 0], [11, 0], [55, 0], [-29, 0], [0, -32], [0, -29], [0, -63], [-11, 0], [36, 0], [0, -54], [0, 49], [10, 0], [0, -1], [0, 24], [0, -14], [33, 0], [4, 0], [36, 0], [-5, 0], [-20, 0], [0, 37], [-40, 0], [4, 0], [0, 52], [5, 0], [-17, 0], [0, 33], [0, -1], [0, -5], [64, 0], [-40, 0], [0, -53], [53, 0], [-52, 0], [0, -2], [0, -77], [30, 0], [0, -29], [0, 6], [0, 1], [-59, 0], [0, 41], [0, 6], [0, 74], [22, 0], [13, 0], [0, 40], [-30, 0], [0, -10], [1, 0], [0, 69], [-16, 0], [42, 0], [0, -70], [-5, 0], [0, -31], [-56, 0], [0, -45], [0, 3], [0, -39], [16, 0], [37, 0], [0, 19], [0, -23], [24, 0], [14, 0], [0, -40], [0, -36], [0, -1], [0, 48], [-28, 0], [22, 0], [0, 10], [-42, 0], [0, 2], [0, 4], [-25, 0], [65, 0], [0, 78], [0, 1], [0, 0], [58, 0], [-27, 0], [0, -42], [0, 70], [26, 0], [10, 0], [24, 0], [-9, 0], [-29, 0], [0, 58], [-58, 0], [0, -33], [-24, 0]]
